# Copyright (C) 2016-2018, 2021 |Meso|Star> (contact@meso-star.com)
#
# This software is a computer program whose purpose is to generate files
# used to build the Star-3D library.
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

cmake_minimum_required(VERSION 3.1)

# Try to find the Clipper devel. Once done this will define:
#   - Clipper_FOUND: system has Clipper
#   - Clipper_INCLUDE_DIR: the include directory
#   - Clipper: Link this to use Clipper

find_path(Clipper_INCLUDE_DIR polyclipping/clipper.hpp)
set(Clipper_INCLUDE_DIR ${Clipper_INCLUDE_DIR}/polyclipping/)

unset(Clipper_LIBRARY CACHE)
unset(Clipper_LIBRARY_DEBUG CACHE)
unset(Clipper_LIBRARY_RELWITHDEBINFO CACHE)
unset(Clipper_LIBRARY_MINSIZEREL CACHE)
find_library(Clipper_LIBRARY polyclipping
  DOC "Path to the clipper library used during release builds.")
find_library(Clipper_LIBRARY_DEBUG polyclipping-dbg
  DOC "Path to the clipper library used during debug builds.")

# Create the imported library target
if(CMAKE_HOST_WIN32)
  set(_property IMPORTED_IMPLIB)
else(CMAKE_HOST_WIN32)
  set(_property IMPORTED_LOCATION)
  if(NOT Clipper_LIBRARY_DEBUG) # Fallback lib
    get_property(_doc CACHE Clipper_LIBRARY_DEBUG PROPERTY HELPSTRING)
    set(Clipper_LIBRARY_DEBUG ${Clipper_LIBRARY} CACHE PATH ${_doc} FORCE)
  endif()
endif()

add_library(Clipper SHARED IMPORTED)
set_target_properties(Clipper PROPERTIES
  ${_property} ${Clipper_LIBRARY_DEBUG}
  ${_property}_DEBUG ${Clipper_LIBRARY_DEBUG}
  ${_property}_RELEASE ${Clipper_LIBRARY})

# Check the package
include(FindPackageHandleStandardArgs)
FIND_PACKAGE_HANDLE_STANDARD_ARGS(Clipper DEFAULT_MSG
  Clipper_INCLUDE_DIR
  Clipper_LIBRARY
  Clipper_LIBRARY_DEBUG)

